@@ -3,11 +3,7 @@ EarthMC is a large Minecraft server this package lets you get info about things
 """
 from __future__ import annotations
 
-from typing import Tuple
-
-from . import util, exceptions
-
-__version__ = "v1.2"
+__version__ = "v1.3"
 __all__ = [
     "__version__",
     "util",
@@ -17,6 +13,10 @@ __all__ = [
     "Nation",
 ]
 
+from typing import Dict, Set, Tuple
+
+from . import util, exceptions
+
 
 class Nation:
     """
@@ -27,11 +27,11 @@ class Nation:
     :raises NationNotFoundException: The nation could not be found
     """
     name: str  #: The name of the nation
-    towns: list[Town]  #: The towns in the nation
+    towns: Set[Town]  #: The towns in the nation
     capital: Town  #: The capital of the nation
     leader: Resident  #: The leader of the nation
     colour: str  #: The colour that the towns in the nation appear on the map. Standard hex colour code
-    citizens: list[Resident]  #: The citizens of the nation
+    citizens: Set[Resident]  #: The citizens of the nation
 
     def __init__(self, name: str, *,
                  data: Tuple[dict, dict] = None):
@@ -43,13 +43,26 @@ class Nation:
             raise exceptions.NationNotFoundException(
                 "The nation {} was not found".format(name))
         self.name = name
-        self.towns = [Town._with_nation(town, data, self) for town in towns]
+        self.towns = {Town._with_nation(town, data, self) for town in towns}
         self.capital = next(
             (town for town in self.towns if town.flags["capital"]))
         self.leader = self.capital.mayor
         self.colour = self.capital.colour
-        self.citizens = [citizen for town in self.towns for citizen in
-                         town.residents]
+        self.citizens = {citizen for town in self.towns for citizen in
+                         town.residents}
+
+    @classmethod
+    def all(cls, *, data: Tuple[dict, dict] = None) -> Set[Nation]:
+        """
+        Returns a set of all nations
+
+        :param tuple[dict,dict] data: Data from :meth:`emc.util.get_data`
+        :return: A set of all nations
+        :rtype: set[emc.Nation]
+        """
+        if data is None:
+            data = util.get_data()
+        return {cls(nation, data=data) for nation in {data[0][town]["desc"][0][:-1].split(" (")[-1] for town in data[0]} if nation != ""}
 
     def __str__(self) -> str:
         return self.name
@@ -75,8 +88,8 @@ class Town:
     nation: Nation  #: The nation the town is in or None if the town is nationless
     colour: str  #: The colour that the town appears on the map. Standard hex colour code
     mayor: Resident  #: The mayor of the town
-    residents: list[Resident]  #: The residents of the town
-    flags: dict[str, bool]  #: The flags of the town. pvp, mobs, explosions, fire, capital
+    residents: Set[Resident]  #: The residents of the town
+    flags: Dict[str, bool]  #: The flags of the town. pvp, mobs, explosions, fire, capital
 
     def __init__(self, name: str, *,
                  data: Tuple[dict, dict] = None):
@@ -87,14 +100,16 @@ class Town:
             raise exceptions.TownNotFoundException(
                 "The town {} could not be found".format(name))
         self.name = data[0][name]["label"]
-        if self.nation is None:
-            self.nation = Nation(
-                data[0][name]["desc"][0][:-1].split("(")[-1],
-                data=data) or None
+        if not hasattr(self, "nation"):
+            nation = data[0][name]["desc"][0][:-1].split("(")[-1]
+            if nation == "":
+                self.nation = None
+            else:
+                self.nation = Nation(nation, data=data)
         self.colour = data[0][name]["fillcolor"]
         self.mayor = Resident._with_town(data[0][name]["desc"][2], data, self)
-        self.residents = [Resident._with_town(person, data, self)
-                          for person in data[0][name]["desc"][4].split(", ")]
+        self.residents = {Resident._with_town(person, data, self)
+                          for person in data[0][name]["desc"][4].split(", ")}
         self.flags = {
             "pvp": data[0][name]["desc"][7] == "pvp: true",
             "mobs": data[0][name]["desc"][8] == "mobs: true",
@@ -110,6 +125,19 @@ class Town:
         town.__init__(name, data=data)
         return town
 
+    @classmethod
+    def all(cls, *, data: Tuple[dict, dict] = None) -> Set[Town]:
+        """
+        Returns a set of all towns
+
+        :param tuple[dict,dict] data: Data from :meth:`emc.util.get_data`
+        :return: A set of all towns
+        :rtype: set[emc.Town]
+        """
+        if data is None:
+            data = util.get_data()
+        return {cls(town, data=data) for town in data[0]}
+
     def __str__(self) -> str:
         return self.name
 
@@ -129,11 +157,10 @@ class Resident:
 
     :param str name: The name of the resident to search for
     :param tuple[dict,dict] data: Data from :meth:`emc.util.get_data`
-    :param town: Internal use only, will be removed in v1.3
     """
     name: str  #: The name of the resident
     online: bool  #: Weather or not the resident is online
-    position: tuple[int, int, int]  #: The position of the resident, (0, 64, 0) if hidden == True
+    position: Tuple[int, int, int]  #: The position of the resident, (0, 64, 0) if hidden == True or None if online == False
     hidden: bool  #: Weather or not the resident can be seen on the map
     town: Town  #: The town that the resident belongs to, None if the resident is townless
     nation: Nation  #: The nation that the resident's town is in, None if the resident is townless or the town nationless
@@ -152,11 +179,13 @@ class Resident:
             self.online = False
             self.position = None
             self.hidden = True
+        if not hasattr(self, "town"):
+            self.town = next((Town(town_name, data=data) for town_name in data[0] if
+                              name in data[0][town_name]["desc"][4]), None)
         if self.town is None:
-            self.town = Town([town_name for town_name in data[0] if
-                              name in data[0][town_name]["desc"][4]][0],
-                             data=data) or None
-        self.nation = self.town.nation or None
+            self.nation = None
+        else:
+            self.nation = self.town.nation
 
     @classmethod
     def _with_town(cls, name, data, town):
@@ -165,6 +194,32 @@ class Resident:
         resident.__init__(name, data=data)
         return resident
 
+    @classmethod
+    def all_online(cls, *, data: Tuple[dict, dict] = None) -> Set[Resident]:
+        """
+        Returns a set of all online players
+
+        :param tuple[dict,dict] data: Data from :meth:`emc.util.get_data`
+        :return: Set of all online players
+        :rtype: set[emc.Resident]
+        """
+        if data is None:
+            data = util.get_data()
+        return {cls(resident, data=data) for person in data[1]["players"] for resident in person["account"]}
+
+    @classmethod
+    def all(cls, *, data: Tuple[dict, dict] = None) -> Set[Resident]:
+        """
+        Returns a set of all players who are in a town
+
+        :param tuple[dict,dict] data: Data from :meth:`emc.util.get_data`
+        :return: Set of all players who are in a town
+        :rtype: set[emc.Resident]
+        """
+        if data is None:
+            data = util.get_data()
+        return {resident for town in Town.all(data=data) for resident in town.residents}
+
     def __str__(self) -> str:
         return self.name
 
